@@ -17,7 +17,7 @@
 #include <KLocalizedString>
 
 SpellCheckRunner::SpellCheckRunner(QObject *parent, const KPluginMetaData &metaData, const QVariantList &args)
-    : Plasma::AbstractRunner(parent, metaData, args)
+    : AbstractRunner(parent, metaData, args)
 {
     setObjectName(QStringLiteral("Spell Checker"));
 }
@@ -45,17 +45,17 @@ void SpellCheckRunner::loadData()
 
     // store all language names, makes it possible to type "spell german TERM" if english locale is set
     // Need to construct a map between natual language names and names the spell-check recognises.
-    const QStringList avail = defaultSpeller->availableLanguages();
+    m_availableLangCodes = defaultSpeller->availableLanguages();
     // We need to filter the available languages so that we associate the natural language
     // name (eg. 'german') with one sub-code.
     QSet<QString> families;
     // First get the families
-    for (const QString &code : avail) {
+    for (const QString &code : std::as_const(m_availableLangCodes)) {
         families += code.left(2);
     }
     // Now for each family figure out which is the main code.
     for (const QString &fcode : std::as_const(families)) {
-        const QStringList family = avail.filter(fcode);
+        const QStringList family = m_availableLangCodes.filter(fcode);
         QString code;
         // If we only have one code, use it.
         // If a string is the default language, use it
@@ -88,6 +88,7 @@ void SpellCheckRunner::destroydata()
 {
     // Clear the data arrays to save memory
     m_spellers.clear();
+    m_availableLangCodes.clear();
 }
 
 void SpellCheckRunner::reloadConfiguration()
@@ -98,8 +99,8 @@ void SpellCheckRunner::reloadConfiguration()
     m_requireTriggerWord = cfg.readEntry("requireTriggerWord", true) && !m_triggerWord.isEmpty();
     m_triggerWord += QLatin1Char(' ');
 
-    Plasma::RunnerSyntax s(i18nc("Spelling checking runner syntax, first word is trigger word, e.g.  \"spell\".", "%1:q:", m_triggerWord),
-                           i18n("Checks the spelling of :q:."));
+    RunnerSyntax s(i18nc("Spelling checking runner syntax, first word is trigger word, e.g.  \"spell\".", "%1:q:", m_triggerWord),
+                   i18n("Checks the spelling of :q:."));
 
     if (!m_requireTriggerWord) {
         s.addExampleQuery(QStringLiteral(":q:"));
@@ -120,14 +121,21 @@ void SpellCheckRunner::reloadConfiguration()
  * Return the empty string if we can't match a language. */
 QString SpellCheckRunner::findLang(const QStringList &terms)
 {
-    const auto &defaultSpeller = m_spellers[QString()];
-    // If first term is a language code (like en_GB), set it as the spell-check language
-    if (!terms.isEmpty() && defaultSpeller->availableLanguages().contains(terms[0])) {
-        return terms[0];
+    if (terms.isEmpty()) {
+        return QString();
     }
+
+    // If first term is a language code (like en_GB, en_gb or en), set it as the spell-check language
+    const auto langCodeIt = std::find_if(m_availableLangCodes.cbegin(), m_availableLangCodes.cend(), [&terms](const QString &languageCode) {
+        return languageCode.startsWith(terms[0], Qt::CaseInsensitive);
+    });
+    if (langCodeIt != m_availableLangCodes.cend()) {
+        return *langCodeIt;
+    }
+
     // If we have two terms and the first is a language name (eg 'french'),
     // set it as the available language
-    else if (terms.count() >= 2) {
+    if (terms.count() >= 2) {
         QString code;
         {
             // Is this a descriptive language name?
@@ -146,9 +154,8 @@ QString SpellCheckRunner::findLang(const QStringList &terms)
 
         if (!code.isEmpty()) {
             // We found a valid language! Check still available
-            const QStringList avail = defaultSpeller->availableLanguages();
             // Does the spell-checker like it?
-            if (avail.contains(code)) {
+            if (m_availableLangCodes.contains(code)) {
                 return code;
             }
         }
@@ -157,7 +164,7 @@ QString SpellCheckRunner::findLang(const QStringList &terms)
     return QString();
 }
 
-void SpellCheckRunner::match(Plasma::RunnerContext &context)
+void SpellCheckRunner::match(RunnerContext &context)
 {
     const QString term = context.query();
     QString query = term;
@@ -173,9 +180,10 @@ void SpellCheckRunner::match(Plasma::RunnerContext &context)
     // Pointer to speller object with our chosen language
     QSharedPointer<Sonnet::Speller> speller = m_spellers[QString()];
 
+    QString lang;
     if (speller->isValid()) {
         QStringList terms = query.split(QLatin1Char(' '), Qt::SkipEmptyParts);
-        const QString lang = findLang(terms);
+        lang = findLang(terms);
         // If we found a language, create a new speller object using it.
         if (!lang.isEmpty()) {
             // First term is the language
@@ -199,44 +207,71 @@ void SpellCheckRunner::match(Plasma::RunnerContext &context)
     }
 
     if (speller->isValid()) {
-        QStringList suggestions;
-        const bool correct = speller->checkAndSuggest(query, suggestions);
-        if (correct) {
-            Plasma::QueryMatch match(this);
-            match.setType(Plasma::QueryMatch::ExactMatch);
-            match.setIconName(QStringLiteral("checkbox"));
-            match.setText(query);
-            match.setSubtext(i18nc("Term is spelled correctly", "Correct"));
-            match.setData(query);
-            context.addMatch(match);
-        } else {
-            for (const auto &suggestion : std::as_const(suggestions)) {
-                Plasma::QueryMatch match(this);
-                match.setType(Plasma::QueryMatch::ExactMatch);
-                match.setIconName(QStringLiteral("edit-rename"));
-                match.setText(suggestion);
-                match.setSubtext(i18n("Suggested term"));
-                match.setData(suggestion);
-                context.addMatch(match);
+        const auto fillMatch = [this, &context, &query, &speller](const QString &langCode = QString()) {
+            if (!langCode.isEmpty()) {
+                speller->setLanguage(langCode);
             }
+
+            QStringList suggestions;
+            const bool correct = speller->checkAndSuggest(query, suggestions);
+
+            if (correct) {
+                QueryMatch match(this);
+                match.setType(QueryMatch::ExactMatch);
+                match.setIconName(QStringLiteral("checkbox"));
+                match.setText(query);
+                match.setSubtext(i18nc("Term is spelled correctly", "Correct"));
+                match.setData(query);
+                context.addMatch(match);
+            } else if (!suggestions.isEmpty()) {
+                for (const auto &suggestion : std::as_const(suggestions)) {
+                    QueryMatch match(this);
+                    match.setType(QueryMatch::ExactMatch);
+                    match.setIconName(QStringLiteral("edit-rename"));
+                    match.setText(suggestion);
+                    match.setSubtext(i18n("Suggested term"));
+                    match.setData(suggestion);
+                    context.addMatch(match);
+                }
+            } else {
+                return false;
+            }
+
+            return true;
+        };
+
+        if (!fillMatch() && lang.isEmpty() && m_availableLangCodes.count() >= 2) {
+            // Perhaps the term is not in the default dictionary, try other dictionaries.
+            const QString defaultLangCode = speller->language();
+            for (const QString &langCode : std::as_const(m_availableLangCodes)) {
+                if (langCode == defaultLangCode) {
+                    continue;
+                }
+
+                if (fillMatch(langCode)) {
+                    // The dictionary returns valid results
+                    break;
+                }
+            }
+            // No need to reset the default language as the speller will be reset in destroydata()
         }
     } else {
-        Plasma::QueryMatch match(this);
-        match.setType(Plasma::QueryMatch::ExactMatch);
+        QueryMatch match(this);
+        match.setType(QueryMatch::ExactMatch);
         match.setIconName(QStringLiteral("data-error"));
         match.setText(i18n("No dictionary found, please install hspell"));
         context.addMatch(match);
     }
 }
 
-void SpellCheckRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
+void SpellCheckRunner::run(const RunnerContext &context, const QueryMatch &match)
 {
     Q_UNUSED(context)
 
     QGuiApplication::clipboard()->setText(match.data().toString());
 }
 
-QMimeData *SpellCheckRunner::mimeDataForMatch(const Plasma::QueryMatch &match)
+QMimeData *SpellCheckRunner::mimeDataForMatch(const QueryMatch &match)
 {
     QMimeData *result = new QMimeData();
     const QString text = match.data().toString();
